/*

Read a symbol table file and create a priority queue from it.
Print the queue in the format defined by the dot program.
Input from stdin or named file, output to stdout.

Doug Johnson, djohnson@cs.washington.edu, February 2002

The dot program is part of the graphics visualization tool set that
is available from ATT.  <http://www.research.att.com/sw/tools/graphviz/>

The format of each row of the input symbol table file is simply

 "<numeric value> <symbol name>"

See symbols.c for more information.

The dot program can be invoked after this program to convert the
output to postscript for plotting.

  ./mainBHeap symbols.txt | dot -Tps -o heap.ps


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "reporter.h"
#include "bheap.h"
#include "plotbheap.h"
#include "symbol.h"

#define MAX_NAME_LENGTH 100

ElementPrintLabel symbolPrintLabel;

int main(int argc,char *argv[]) {

  FILE *src;			/* input FILE */
  int count;			/* number of symbols provided */
  
  PriorityQueue H;		/* a PriorityQueue */

  struct Symbol *symP;		/* pointer to a Symbol struct */
  struct Symbol **symTable;	/* array of pointers to Symbol structs */
  struct Symbol MIN_ELEMENT = {	/* absolutely first element */
    "",				/* zero length name */
    0				/* zero value */
  };

  /* read from stdin or the named file */

  if (argc == 1) {
    src = stdin;
  } else if (argc == 2) {
    src = fopen(argv[1],"r");
    if (src == NULL) {
      printf("main: file open error on %s.\n",argv[1]);
      return 1;
    }
  } else {  
    printf("usage: %s [<filename>]\n",argv[0]);
    return 1;
  }

  /* Read the symbol definitions. */

  symTable = readSymbols(src);
  fclose(src);

  /* Build the heap, one symbol at a time */

  H = CreatePriorityQueue(500,&MIN_ELEMENT,symbolCompareByName);
  count = 0;
  while (symTable[count] != NULL) {
    InsertElement(H,symTable[count]);
    count++;
  }

  /* Print out a description of the heap in dot format */

  plotBHeapHeader(H,"Binary Heap - name priority");
  plotBHeapNodes(H,symbolPrintLabel);
  plotBHeapEdges(H);
  plotBHeapFooter(H);

  /* Delete everything all the heap entries, one entry at a time */

  while ((symP = DeleteMinElement(H)) != NULL) {
    printf("/* deleted ");
    symbolPrintLabel(symP);
    printf(" */\n");
  }

  /* Clean up */

  DestroyPriorityQueue(H);

  freeSymbols(symTable);

  return 0;
}

void symbolPrintLabel(void *E) {
  if (E==NULL)
    printf("<NULL>");
  else
    printf("%s\\n%lx",((struct Symbol *)E)->name,((struct Symbol *)E)->value);
}
